from __future__ import division, print_function

import random

import tools


class Card(object):
    def __init__(self, name, image, strengths):
        self.name = name
        self.image = tools.load_image(image)
        self.strengths = strengths
        self.special = False

    def beats(self, other):
        return (other.name in self.strengths)


CARDS = {
    "rock": Card("rock", "rock.png", ["scissors", "glue"]),
    "paper": Card("paper", "paper.png", ["rock", "rubber"]),
    "scissors": Card("scissors", "scissors.png", ["paper"]),
    "rubber": Card("rubber", "rubber.png",
                   ["rock", "scissors", "glue", "water"]),
    "glue": Card("glue", "glue.png", ["scissors", "paper", "water"]),
    "water": Card("water", "water.png", ["rock", "paper", "scissors", "fire"]),
    "fire": Card("fire", "fire.png",
                 ["rock", "paper", "scissors", "rubber", "glue"]),
    }


class Deck(object):
    def __init__(self, card_data):
        self.card_data = card_data
        self.cards = []
        self.graveyard = []
        self.reset()

    def reset(self):
        self.cards = []
        self.graveyard = []
        for card,num in self.card_data.iteritems():
            cards = [CARDS[card]]*num
            self.cards.extend(cards)
        self.shuffle()

    def shuffle(self):
        random.shuffle(self.cards)

    def draw_card(self):
        if not self.cards:
            self.necromancy()
        return self.cards.pop()

    def release_card(self, card):
        self.graveyard.append(card)

    def necromancy(self):
        self.cards = self.graveyard
        self.graveyard = []
        self.shuffle()

DECKS = {
    "primary": {
        "rock": 24,
        "paper": 24,
        "scissors": 24,
        },

    "secondary": {
        "rock": 16,
        "paper": 16,
        "scissors": 16,
        "rubber": 7,
        "glue": 9
        },

    "tertiary": {
        "paper": 13,
        "rock": 13,
        "scissors": 13,
        "rubber": 7,
        "glue": 10,
        "water": 8,
        },

    "quaternary": {
        "paper": 12,
        "rock": 12,
        "scissors": 12,
        "rubber": 7,
        "glue": 10,
        "water": 7,
        "fire": 4,
        }
    }

DECKORDER = [("Rock-Paper-Scissors", "primary"),
             ("Rubber and Glue", "secondary"),
             ("Water", "tertiary"),
             ("Fire", "quaternary")]

def get_deck(name):
    return Deck(DECKS[name])
