from __future__ import division

import pygame
from pygame.locals import *

import es
import events
import factory
import render
import scribe


class GuiManager():
    def __init__(self, world):
        self.world = world
        self.cursor = self.make_cursor()
        self.active_item = None
        self.items = []
        self.title = None

    def make_cursor(self):
        entity = factory.sticker("cursor.png", (0,0))
        self.world.add_entity(entity)
        entity.deactivate()
        return entity

    def make_title(self, title, position):
        entity = factory.label(position, title, size=32, colour=(0,0,0))
        self.world.add_entity(entity)
        return entity

    def add_item(self, text, function, position=None):
        if position is None:
            if not self.items:
                position = (0,0)
            else:
                x,y = self.items[-1]["rect"].midbottom
                position = (x, y+10)
        item = es.Entity()
        image = scribe.label(text, size=32)
        item["image"] = render.Drawable(image, "fg")
        item["rect"] = rect = image.get_rect()
        item["callback"] = function
        rect.midtop = position
        self.items.append(item)
        self.world.add_entity(item)

    def clear_items(self):
        for item in self.items:
            item.die()
        if self.title:
            self.title.die()
        self.items = []
        self.active_item = None

    def set_menu(self, position, items, title=None):
        self.clear_items()
        if title:
            self.title = self.make_title(title, position)
        x,y = position
        head = items[0]
        self.add_item(head[0], head[1], (x, y+16))
        for text,func in items[1:]:
            self.add_item(text, func)
        self.select(0)

    def select(self, index):
        self.active_item = active = self.items[index]
        self.cursor["rect"].midright = active["rect"].midleft
        self.cursor.activate()

    def move_sel(self, direction):
        if self.active_item:
            active_index = self.items.index(self.active_item)
            active_index += direction
            if 0 <= active_index < len(self.items):
                self.select(active_index)

    def confirm(self):
        if self.active_item:
            self.active_item["callback"]()


class GuiSystem(object):
    def __init__(self, master):
        self.master = master

    def run(self):
        for event in events.get():
            if event.kind == "key":
                if event.value == K_UP:
                    self.master.gui.move_sel(-1)
                elif event.value == K_DOWN:
                    self.master.gui.move_sel(1)
                elif event.value in [K_SPACE, K_RETURN]:
                    self.master.gui.confirm()
